package client;

import java.util.ArrayList;
import java.util.List;

import simulation.Constants;
import client.PeerState.Neighbor;
import client.messages.ChunkReply;
import client.messages.ChunkRequest;

class DownloadSession {
	private Neighbor neighbor;
	final String source;
	final int piece;
	private long lastUpdated;
	private List<Integer> current;

	DownloadSession(String source, int piece) {
		this.piece = piece;
		this.source = source;
		this.current = new ArrayList<Integer>();

		for (int i = 0; i < Math.min(Constants.BATCH, Constants.NUM_OF_CHUNKS); i++)
			current.add(i);

		update();
	}

	private void update() {
		this.lastUpdated = System.currentTimeMillis();
	}

	Neighbor neighbor() {
		return neighbor;
	}

	ChunkRequest start(Neighbor n) {
		assert n != null;
		assert current.size() > 0;

		update();
		this.neighbor = n;
		this.neighbor.upload.ensureStarted();

		ChunkRequest result = new ChunkRequest(piece, current.get(0), source);
		for (Integer i : current)
			result.chunkIndex.add(i);

		return result;
	}

	ChunkRequest proceed(ChunkReply cr) {
		assert cr.pieceIndex == this.piece;

		update();
		this.neighbor.upload.report();

		current.remove(new Integer(cr.chunkIndex));

		if (!finished()) {
			int max = 0;
			for (Integer i : current) {
				if (i > max)
					max = i;
			}
			int index = max + 1;
			if (index < Constants.NUM_OF_CHUNKS) {
				current.add(index);
				return new ChunkRequest(piece, index, source);
			}
		}

		return null;
	}

	boolean finished() {
		return current.size() == 0;
	}

	boolean stale() {
		assert neighbor != null;
		return (System.currentTimeMillis() > lastUpdated
				+ Constants.DOWNLOAD_SESSION_TIMEOUT)
				|| ((System.currentTimeMillis() > lastUpdated
						+ Constants.FAST_SESSION_TIMEOUT) && neighbor.peer_choking);
	}
}
