package client;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import util.ThreadUtil;

/**
 * TODO: locks on state components
 * 
 * @author kuat
 * 
 */
public class GUI extends JPanel implements Runnable {
	private static final long serialVersionUID = 4760910353729992014L;
	private final List<Peer> peers;
	private final StyledDocument doc;

	public GUI() {
		peers = Collections.synchronizedList(new ArrayList<Peer>());

		setLayout(new BorderLayout());

		// Create a text pane.
		JTextPane textPane = new JTextPane();
		textPane.setDoubleBuffered(true);
		doc = textPane.getStyledDocument();
		addStylesToDocument(doc);
		JScrollPane paneScrollPane = new JScrollPane(textPane);
		paneScrollPane
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		paneScrollPane.setPreferredSize(new Dimension(1200, 600));
		paneScrollPane.setMinimumSize(new Dimension(10, 10));

		// Put the editor pane and the text pane in a split pane.

		add(paneScrollPane, BorderLayout.CENTER);

		// Create and set up the window.
		JFrame frame = new JFrame("Peers");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Add content to the window.
		frame.add(this);

		// Display the window.
		frame.pack();
		frame.setVisible(true);

		new Thread(this).start();
	}

	public void run() {
		while (true) {
			final List<String> output1 = new ArrayList<String>();
			final List<String> output2 = new ArrayList<String>();

			synchronized (peers) {
				for (Peer peer : peers) {
					PeerState state = peer.state();
					StringBuilder sb = new StringBuilder();
					Set<Integer> incomplete = state.file().incomplete();

					sb.append("[");
					for (int i = 0; i < state.file().numberPieces(); i++) {
						String s = null;
						;
						if (incomplete.contains(i)) {
							int j = state.file().getPiece(i).completed();
							s = new Integer(j).toString();
						} else
							s = "+++";
						sb.append(format(s, 3));
					}

					sb.append("]");
					output1.add(sb.toString());

					sb = new StringBuilder();
					sb.append(format(state.name(), 5)).append(" ").append(
							peer.statistics()).append(" ").append(
							state.getNeighbors()).append(" ");

					if (state.file().isComplete())
						sb.append(state.file().content().hashCode());

					sb.append("\n");

					output2.add(sb.toString());
				}
			}

			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					try {
						doc.remove(0, doc.getLength());
						for (int i = 0; i < output1.size(); i++) {
							doc.insertString(doc.getLength(), output1.get(i),
									doc.getStyle("color"));
							doc.insertString(doc.getLength(), output2.get(i),
									doc.getStyle("large"));
						}
					} catch (BadLocationException ble) {
						System.err
								.println("Couldn't insert initial text into text pane.");
					}
				}

			});
			ThreadUtil.sleep();
		}

	}

	protected void addStylesToDocument(StyledDocument doc) {
		// Initialize some styles.
		Style def = StyleContext.getDefaultStyleContext().getStyle(
				StyleContext.DEFAULT_STYLE);

		Style regular = doc.addStyle("regular", def);
		StyleConstants.setFontFamily(def, "Monospaced");

		Style s = doc.addStyle("large", regular);
		StyleConstants.setBold(s, true);
		StyleConstants.setFontSize(s, 16);

		Style green = doc.addStyle("color", s);
		StyleConstants.setForeground(green, Color.BLUE);
	}

	public void add(Peer peer) {
		synchronized(peers) {
			peers.add(peer);
		}
	}

	private String format(String s, int i) {
		if (s == null)
			s = "";
		StringBuilder sb = new StringBuilder(s);
		while (sb.length() < i)
			sb.append(" ");
		return sb.toString();
	}

}
