package client;

import java.util.ArrayList;
import java.util.List;

import simulation.Constants;

public class Rate {
	private final List<Long> received;
	private long startup;

	Rate() {
		received = new ArrayList<Long>();
		startup = -1;
	}

	public void ensureStarted() {
		synchronized (received) {
			if (startup == -1)
				startup = System.currentTimeMillis();
		}
	}

	public double measure() {
		synchronized (received) {
			if (startup == -1)
				return 0;

			long now = System.currentTimeMillis();
			// remove stale records
			while (received.size() > 0
					&& received.get(0) < now - Constants.UPLOAD_AVERAGE_PERIOD)
				received.remove(0);

			double result = received.size();
			if (now > startup + Constants.UPLOAD_AVERAGE_PERIOD) {
				return result / Constants.UPLOAD_AVERAGE_PERIOD;
			} else {
				return result / (now - startup);
			}
		}
	}

	public void report() {
		synchronized (received) {
			assert startup != -1;

			// prevent buffer overflow
			if (received.size() < 10000)
				received.add(System.currentTimeMillis());
		}
	}
}
