package client;

import java.util.ArrayList;
import java.util.List;

import simulation.Constants;

class Statistics {

	private final List<Long> forwarded;
	private final List<Long> received;
	private final List<Long> sent;
	
	Statistics() {
		this.sent = new ArrayList<Long>();
		this.received = new ArrayList<Long>();
		this.forwarded = new ArrayList<Long>();
	}
	
	private int count(List<Long> list) {
		synchronized (list) {
			long now = System.currentTimeMillis();
			while (list.size() > 0 && list.get(0) < now - Constants.MOVING_AVERAGE_PERIOD)
				list.remove(0);
			return list.size();
		}
	}
	
	private void report(List<Long> list) {
		synchronized (list) {
			// prevent buffer overflow
			if (list.size() < 1000)
				list.add(System.currentTimeMillis());
		}
	}
	
	@Override
	public String toString() {
		int a = count(forwarded);
		int b = count(received);
		int c = count(sent);
		StringBuilder sb = new StringBuilder();
		double k = Constants.MOVING_AVERAGE_PERIOD / 1000;
		sb.append(" forwarded: ").append(((double) a) / k);
		sb.append(" sent: ").append(((double) b) / k);
		sb.append(" received: ").append(((double) c) / k);
		return sb.toString();		
	}
	
	void forwarded() {
		report(forwarded);
	}
	
	void received() {
		report(received);
	}
	
	void sent() {
		report(sent);
	}

}
