package client;

import java.util.HashMap;
import java.util.Map;

import simulation.Constants;

import common.Server;

public class Timer {

	private final Map<Peer, Long> started;
	private final Map<Peer, Long> finished;
	private final Map<Peer, Integer> initial;

	private Timer() {
		started = new HashMap<Peer, Long>();
		finished = new HashMap<Peer, Long>();
		initial = new HashMap<Peer, Integer>();
	}

	public final static Timer singleton = new Timer();

	public void register(Peer p) {
		synchronized (singleton) {
			started.put(p, System.currentTimeMillis());
			initial.put(p, p.state().file().numberPieces()
					- p.state().file().incomplete().size());
		}
	}

	public void finished(Peer peer) {
		synchronized (singleton) {
			if (!finished.containsKey(peer)) {
				finished.put(peer, System.currentTimeMillis());
			}

			if (finished.keySet().size() == started.keySet().size()) {
				int n = peer.state().file().numberPieces();

				System.err.println("PIECES " + Constants.SIM_NUM_PIECES);
				System.err.println("FORWARDING " + Constants.FORWARDING);
				System.err.println("NEIGHBORS " + Constants.NEIGHBOURS_REPLY);
				System.err.println("LATENCY " + Constants.LATENCY);
				System.err.println("PARALLEL SESSIONS " + Constants.PARALLEL_SESSIONS);
				System.err.println("TCP TRACKER " + Constants.USE_TCP);
				System.err.println("EMULATOR " + Server.emulator());
				
				for (Peer p : started.keySet()) {
					System.err.println(p.state().name() + " @ " + p.state().port() + " , "
							+ (finished.get(p) - started.get(p)) + " , "
							+ (((double) initial.get(p)) / n));
				}
				
				System.exit(1);

			}
		}
	}
}
