/**
 * 
 */
package client.messages;

import java.util.HashSet;
import java.util.Set;

/**
 * @author turalb
 * 
 * The request message is fixed length, and is used to request a block. 
 * The payload contains the following information:
 * 
 * 	index: integer specifying the zero-based piece index
 * 	begin: integer specifying the zero-based byte offset within the piece
 * 	length: integer specifying the requested length. 
 *
 */
public class ChunkRequest extends Message {
	
	private static final long serialVersionUID = 1865354232900559139L;
	/** integer specifying the zero-based piece index */
	public int pieceIndex;	
	/** integer specifying the zero-based byte offset within the piece */ 
	public Set<Integer> chunkIndex;	

	public ChunkRequest() {}

	/**
	 * Requests a chunk
	 * 
	 * @param pieceIndex - 	the index of the piece containing the desired 
	 * 						chunk
	 * @param chunkIndex -	the index of the chunk within that piece
	 */
	public ChunkRequest(int pieceIndex, int chunkIndex, String source) {
		this.pieceIndex = pieceIndex;
		this.chunkIndex = new HashSet<Integer>();
		this.chunkIndex.add(chunkIndex);
		this.sourceName = source;
	}
	
	@Override
	public String toString() {
		return "[" + " request chunk " + chunkIndex + " in piece " + pieceIndex + "]";
	}
	
//	/**
//	 * @param index
//	 * Sets the index field to index
//	 */
//	public void setIndex(int index) {
//		this.pieceIndex = index;
//	}
	
//	/**
//	 * 
//	 * @param begin
//	 * Sets the begin field to begin
//	 */
//	public void setBegin(int begin) {
//		this.chunkIndex = begin;
//	}
	
//	/**
//	 * @param length
//	 * Sets the length field to length
//	 */
//	public void setLength(int length) {
//		this.length = length;
//	}
//
}
