package common;

import java.io.Serializable;

/**
 * Address of the peer.
 * 
 * @author kuat
 *
 */
public class PeerAddress {
	
	/**
	 * Serializable class of the address.
	 * 
	 * @author kuat
	 *
	 */
	public static class PeerAddressSerial implements Serializable {
		private static final long serialVersionUID = 1980418652062145254L;
		public int port;
		public String host;
	}
	
	private final int port;
	private final String host;
	
	public PeerAddress(int tcp, String host) {
		this.port = tcp;
		this.host = host;
	}
	
	public PeerAddress(PeerAddressSerial ser) {
		this.port = ser.port;
		this.host = ser.host;
	}
	
	public PeerAddressSerial serializableView() {
		PeerAddressSerial ser = new PeerAddressSerial();
		ser.port = port;
		ser.host = host;
		return ser;
	}

	public int port() {
		return port;		
	}
	
	public String host() {
		return host;
	}
	
	@Override
	public String toString() {
		return "[TCP: " + port + "; HOST: " + host + "]";
	}
	
	@Override
	public boolean equals(Object o) {
		if (!(o instanceof PeerAddress))
			return false;
		
		PeerAddress pa = (PeerAddress) o;
		return this.port == pa.port && this.host.equals(pa.host); 
	}	
	
	@Override 
	public int hashCode() {
		return port;
	}
}
