package common;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.InetSocketAddress;

/**
 * UDP server. Forks a thread that listens on the given port.
 * 
 * @author kuat
 * 
 */
public class UDPServer implements Runnable {
	private final DatagramSocket socket;
	private final Dispatcher dispatcher;
	private final int port;
		
	public UDPServer(int port, Dispatcher dispatcher) throws IOException {
		this(new InetSocketAddress(port), dispatcher);
	}

	private UDPServer(InetSocketAddress port, Dispatcher dispatcher) throws IOException {
		assert dispatcher != null;
		
		this.socket = new DatagramSocket(port);
		this.dispatcher = dispatcher;
		this.port = socket.getPort();
		
		new Thread(this).start();
	}
	
	public UDPServer(Dispatcher dispatcher) throws IOException {
		assert dispatcher != null;
		
		this.socket = new DatagramSocket();
		this.dispatcher = dispatcher;
		this.port = socket.getPort();
		
		new Thread(this).start();
	}

	public void run() {
		DatagramPacket packet;
		
		while (true) {
			try {
				byte[] buf = new byte[1024 * 5];

				packet = new DatagramPacket(buf, buf.length);

				// blocking
				socket.receive(packet);
			
				int port = packet.getPort();				
				dispatcher.process(packet.getData(), packet.getAddress().getHostName(), port);
								
			} catch (IOException e) {
				e.printStackTrace();			
			}
		}
	}
		
	public void send(byte[] buf, String address, int port) {
		assert buf != null : "empty message";
		
		try {
			// send the response to the client at "address" and "port"
			DatagramPacket packet = new DatagramPacket(buf, buf.length, InetAddress.getByName(address), port);				
			socket.send(packet);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public int port() {
		return port;
	}
}
