package simulation;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import tracker.TCPTracker;
import tracker.UDPTracker;
import util.File;
import client.GUI;
import client.Peer;

/***
 * The controller of the simulation.
 * <ol>
 * <li> forks a tracker;
 * <li> forks threads for peers;
 * <li> defines the network delay;
 * <li> collects and presents statistics.
 * </ol>
 * @author kuat, albertni
 *
 */
public class Manager {
	
	private static final int NUM_REPEAT = simulation.Constants.MULTIPLIER;

	public static final String[] CHUNKS_OF_LETTERS = new String[] {
		repeat('a', NUM_REPEAT),
		repeat('b', NUM_REPEAT),
		repeat('c', NUM_REPEAT),
		repeat('d', NUM_REPEAT),
		repeat('e', NUM_REPEAT),
		repeat('f', NUM_REPEAT),
		repeat('g', NUM_REPEAT),
		repeat('h', NUM_REPEAT),
		repeat('i', NUM_REPEAT),
		repeat('j', NUM_REPEAT),
		repeat('k', NUM_REPEAT),
		repeat('l', NUM_REPEAT),
		repeat('m', NUM_REPEAT),
		repeat('n', NUM_REPEAT),
		repeat('o', NUM_REPEAT),
		repeat('p', NUM_REPEAT),
		repeat('q', NUM_REPEAT),
		repeat('r', NUM_REPEAT),
		repeat('s', NUM_REPEAT),
		repeat('t', NUM_REPEAT),
		repeat('u', NUM_REPEAT),
		repeat('v', NUM_REPEAT),
		repeat('w', NUM_REPEAT),
		repeat('x', NUM_REPEAT),
		repeat('y', NUM_REPEAT),
	};
	
	public static void main(String[] args) {

//		if (CHUNKS_OF_LETTERS == null) {
//			System.out.println("is null");
//		}
//		else {
//			System.out.println("not null");
//		}		
		GUI gui = Constants.GUI ? new GUI() : null;
		
		Set<Peer> clientSet = new HashSet<Peer>();
		Set<Integer> portsSet = new HashSet<Integer>();
		
		Random rand = new Random();
		while (portsSet.size() < simulation.Constants.NUM_CLIENTS) {
			int i = rand.nextInt(60000) + 4000;
			portsSet.add(i);			
		}
		
		if (Constants.USE_TCP) 
			new TCPTracker(simulation.Constants.TRACKER_PORT);
		else
			new UDPTracker(simulation.Constants.TRACKER_PORT);
		
		File FILE = File.generateRandomFile(Constants.SIM_NUM_PIECES, "FILE_10");
		
		List<Integer> portsList = new ArrayList<Integer>(portsSet);
		
		// If this is true, starts the network with one seeder and a bunch
		// of empty clients.
		//
		// Otherwise, starts the network with two seeders and a bunch of
		// partly completed clients

		boolean USE_SIMPLE_CASE = false; 
		
		if (USE_SIMPLE_CASE) {
			for (int i = 1; i < simulation.Constants.NUM_CLIENTS; i++) {
				Peer tempPeer = new Peer(portsList.get(i), simulation.Constants.TRACKER_HOST, simulation.Constants.TRACKER_PORT, FILE.getEmptyFile());
				clientSet.add(tempPeer);
			}
			
			// make a seeder
			Peer seeder = new Peer(portsList.get(0), simulation.Constants.TRACKER_HOST, simulation.Constants.TRACKER_PORT, FILE);
			clientSet.add(seeder);
		}
		else {
			// first, give all but two clients a random subpart of the file, so
			// they essentially become peers for the file
			for (int i = 2; i < simulation.Constants.NUM_CLIENTS; i++) {
	//			File randSubPart = new File(FILE_10.getNumOfPieces(), FILE_10.getName());
	//			File[] fArray = new File[] {randSubPart};
	//			Peer tempPeer = new Peer(portsList.get(i), TRACKER_PORT, fArray);
				Peer tempPeer = Peer.generatePeer(portsList.get(i), simulation.Constants.TRACKER_HOST, simulation.Constants.TRACKER_PORT, FILE);
//				clientsInTheNetwork.add(tempPeer);
				clientSet.add(tempPeer);
			}
			
			// make two clients seeders
			for (int i = 0; i < 2; i++) {			
				Peer seeder = new Peer(portsList.get(i), simulation.Constants.TRACKER_HOST, simulation.Constants.TRACKER_PORT, new File[] {FILE});
//				clientsInTheNetwork.add(seeder);
				clientSet.add(seeder);
			}
		}
		
		for (Peer client : clientSet) {
			if (Constants.GUI) gui.add(client);
		}
	}
	
	private static String repeat(char c, int n) {
		char[] charArray = new char[n];
		for (int i = 0; i < n; i++) {
			charArray[i] = c;
		}
		return new String(charArray);
	}
	
	/**
	 * @param n
	 * @return A String of "z"s of length n
	 */
	public static String filler(int n) {
		StringBuffer sb = new StringBuffer("");
		for (int i = 0; i < n; i++) {
			sb.append("z");
		}
		return sb.toString();
	}
	
}