package tracker;

import java.io.Serializable;

import tracker.TrackerState.PeerInfo;

import common.Dispatcher;

public abstract class AbstractTracker implements Dispatcher {

	protected final TrackerState state;

	protected AbstractTracker() {
		this.state = new TrackerState();
	}

	public Serializable process(Object o, String host) {
		if (o instanceof JoinRequest) {
			JoinRequest req = (JoinRequest) o;
			JoinReply reply = new JoinReply();
			
			reply.name = state.register(req.port, req.udp, host);
			state.register(reply.name, req.fileId);
			
			return reply;
		} else if (o instanceof PeerRequest) {
			PeerRequest req = (PeerRequest) o;
			PeerReply reply = new PeerReply();
			
			reply.peers = state.peers(req.name, req.fileId);
			
			return reply;
		} else if (o instanceof CircuitRequest){
			CircuitRequest req = (CircuitRequest) o;
			
			PeerInfo info = state.getRandomAddress(req.from, req.to);
			assert info != null : "no such peer is registered";
			
			return info.addr.serializableView();
		} else if (o instanceof IpRequest) {
			IpRequest req = (IpRequest) o;
			
			PeerInfo info = state.getAddress(req.name);
			assert info != null : "no such peer is registered";
			
			return info.addr.serializableView();
			
		} else {
			return null;
		}
	}

}