package tracker;

import java.io.IOException;

import util.Serializer;

import common.PeerAddress;
import common.PeerAddress.PeerAddressSerial;

public abstract class TrackerClient {

	protected abstract byte[] send(byte[] buf) throws IOException;

	/*
	 * (non-Javadoc)
	 * 
	 * @see tracker.TrackerClient#connect(java.lang.String, int, int)
	 */
	synchronized public JoinReply connect(String fileId, int localPort, int udp)
			throws IOException {
		JoinRequest req = new JoinRequest();
		req.fileId = fileId;
		req.port = localPort;
		req.udp = udp;

		Object o = Serializer.read(send(Serializer.write(req)));
		assert o instanceof JoinReply;
		return (JoinReply) o;
	}

	synchronized public PeerReply peers(String name, String fileId)
			throws IOException {
		PeerRequest req = new PeerRequest();
		req.name = name;
		req.fileId = fileId;		

		Object o = Serializer.read(send(Serializer.write(req)));
		assert o instanceof PeerReply;
		return (PeerReply) o;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see tracker.TrackerClient#circuit(java.lang.String)
	 */
	synchronized public PeerAddress circuit(String from, String to) throws IOException {
		CircuitRequest req = new CircuitRequest();
		req.to = to;
		req.from = from;

		Object o = Serializer.read(send(Serializer.write(req)));
		assert o instanceof PeerAddressSerial;
		return new PeerAddress((PeerAddressSerial) o);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see tracker.TrackerClient#ip(java.lang.String)
	 */
	synchronized public PeerAddress ip(String name) throws IOException {
		IpRequest req = new IpRequest();
		req.name = name;

		Object o = Serializer.read(send(Serializer.write(req)));
		assert o instanceof PeerAddressSerial;
		return new PeerAddress((PeerAddressSerial) o);
	}
}