package tracker;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketTimeoutException;
import java.util.Random;

/**
 * Talks to the tracker. Since we use UDP, the calls to this class are all
 * blocking.
 * 
 * @author kuat
 * 
 */
public class UDPTrackerClient extends TrackerClient {
	public final static int TIMEOUT = 200;
	private final DatagramSocket socket;
	private final InetAddress localhost;
	private final int trackerPort;

	public UDPTrackerClient(String host, int trackerPort) throws IOException {
		this.socket = new DatagramSocket();
		this.localhost = InetAddress.getByName(host);
		this.trackerPort = trackerPort;
	}

	protected final byte[] send(byte[] buf) throws IOException {
		Random r = new Random();
		int timeout = TIMEOUT + r.nextInt(TIMEOUT);
		while (true) {
			// exponential back-off
			try {
				DatagramPacket packet = new DatagramPacket(buf, buf.length,
						localhost, trackerPort);
				socket.send(packet);
				socket.setSoTimeout(timeout);

				byte[] result = new byte[1024 * 10];
				DatagramPacket incoming = new DatagramPacket(result,
						result.length);
				socket.receive(incoming);
				return incoming.getData();
			} catch (SocketTimeoutException e) {
				System.err.println("Tracker request timed out");
				timeout = timeout > 3000 ? timeout : timeout * 2;				
			}
		}
	}
}
